float HueToRGB(float f1, float f2, float hue)
{
	if (hue < 0.0)
		hue += 1.0;
	else if (hue > 1.0)
		hue -= 1.0;
	float res;
	if ((6.0 * hue) < 1.0)
		res = f1 + (f2 - f1) * 6.0 * hue;
	else if ((2.0 * hue) < 1.0)
		res = f2;
	else if ((3.0 * hue) < 2.0)
		res = f1 + (f2 - f1) * ((2.0 / 3.0) - hue) * 6.0;
	else
		res = f1;
	return res;
}

vec3 HSLToRGB(vec3 hsl)
{
	vec3 rgb;
	
	if (hsl.y == 0.0)
		rgb = vec3(hsl.z); // Luminance
	else
	{
		float f2;
		
		if (hsl.z < 0.5)
			f2 = hsl.z * (1.0 + hsl.y);
		else
			f2 = (hsl.z + hsl.y) - (hsl.y * hsl.z);
			
		float f1 = 2.0 * hsl.z - f2;
		
		rgb.r = HueToRGB(f1, f2, hsl.x + (1.0/3.0));
		rgb.g = HueToRGB(f1, f2, hsl.x);
		rgb.b = HueToRGB(f1, f2, hsl.x - (1.0/3.0));
	}
	
	return rgb;
}

void main() {
	vec3 colour = texture(InputTexture, TexCoord).rgb;
	if (colour.g > 0.6 && colour.r < 0.26 && colour.b < 0.38)	//acts only if the pixel is green
	{	float hue = colour.r * 4;
		float lum = colour.b * 2.7;
		float sat = 1;
		if (desat == 1) { sat -= lum;	//desaturates the highlights of the texture
		}
		vec3 mire = HSLToRGB(vec3(hue, sat, lum));	//reconstructs the vanilla texture
		
		vec2 xy = TexCoord;
		if (warp > 0)
		{	//xy = ((xy - 0.5) * (lum * 6)) + 0.5;	//warping effect for mirror
			float angle = radians(lum * 360);
			float xdist = cos(angle);
			float ydist = sin(angle);
			xy += vec2(xdist, ydist) * warp;
			xy.x -= warp * 0.75;
		}
		xy.y = (1.0 - TexCoord.y) + pitch;	//flips the screen's y coordinate and adjusts the horizontal position using pitch
		vec3 mirror = texture(InputTexture, xy).rgb;	//finds the colour stored at the y flipped coordinate
		
		if (mirror.g > 0.6 && mirror.r < 0.26 && mirror.b < 0.38)
		{	mire *= clamp((1 - balance) * 2, 0.0, 1.0);
			FragColor = vec4(sqrt(mire * mire * 0.5), 1.0);
		} else
		{	if (shading == 1) { mirror *= pow(lum, 0.25);	//adds luminosity to the reflection
			}
			if (letterbox == 1) { mirror *= 1.0 - (abs(TexCoord.y - 0.5) * 2);	//mirror fades at top and bottom
			}
			if (angle == 1) { mirror *= 1.0 - clamp(abs(pitch * ((TexCoord.y - pitch - 0.5) * 2)), 0.0, 1.0);	//mirror progressively vanishes while looking up or down
			}
			mire *= clamp((1 - balance) * 2, 0.0, 1.0);
			mirror *= clamp(balance * 2, 0.0, 1.0);
			vec3 merge = sqrt(((mire * mire) + (mirror * mirror)) * 0.5);	//blends the reflection and texture together
			FragColor = vec4(merge, 1.0);
		}
	}
	else { FragColor = vec4(colour, 1.0);
	}
}

//Thanks to Romain Dura for the colour blending code:

/*
** Copyright (c) 2012, Romain Dura romain@shazbits.com
** 
** Permission to use, copy, modify, and/or distribute this software for any 
** purpose with or without fee is hereby granted, provided that the above 
** copyright notice and this permission notice appear in all copies.
** 
** THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES 
** WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF 
** MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY 
** SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
** WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN 
** ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR 
** IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
*/